<?php

namespace Drupal\bazaarvoice_productfeed\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Database\Database;
use Drupal\Component\Utility\Html;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Class AttributesAddForm.
 *
 * @package Drupal\bazaarvoice_productfeed\Form
 */
class AttributesAddForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'bazaarvoice_productfeed_attributes_add_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $values = $form_state->getValues();

    $form['attribute_label'] = [
      '#type' => 'textfield',
      '#title' => t('Label'),
      '#required' => TRUE,
      '#description' => t('The human readable label'),
      '#default_value' => isset($values['attribute_label']) ? $values['attribute_label'] : '',
    ];
    $form['attribute_id'] = [
      '#type' => 'textfield',
      '#title' => t('Attribute ID'),
      '#required' => TRUE,
      '#description' => t('The ID value to be used in the Product Feed. Once set this cannot be changed.'),
      '#default_value' => isset($values['attribute_id']) ? $values['attribute_id'] : '',
    ];
    $field_options = ['text' => $this->t('Text')];

    // Get list of all field types.
    $field_type_definitions = \Drupal::service('plugin.manager.field.field_type')
      ->getDefinitions();
    foreach ($field_type_definitions as $field_type_key => $field_type) {
      if (!isset($field_options[$field_type_key])) {
        $field_options[$field_type_key] = $field_type['label'];
      }
    }

    $form['field_types'] = array(
      '#type' => 'checkboxes',
      '#title' => t('Field Types'),
      '#required' => TRUE,
      '#description' => t('What field types can be mapped to this attribute'),
      '#options' => $field_options,
    );
    // Have a value passed?
    if (isset($values['field_types'])) {
      $form['field_types']['#default_value'] = $values['field_types'];
    }
    // Have field options?
    if (count($field_options) == 0) {
      $form['field_types']['#type'] = 'markup';
      $form['field_types']['#value'] = t('There are no field types available.');
    }

    $form['actions']['#type'] = 'actions';
    $form['actions']['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Save Attribute'),
    );

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();

    // Check that the label is a plain text label.
    $label = Html::escape(trim(strip_tags($values['attribute_label'])));
    // Have a valid label?
    if (!$label) {
      $form_state->setErrorByName('attribute_label', t('Invalid label provided'));
    }
    // Check that the ID is not already in use.
    $attribute_id = Html::escape(trim(strip_tags($values['attribute_id'])));
    // @TODO: some string formatting for ID?
    if ($attribute_id) {
      // Check DB to see if ID already exists.
      $connection = Database::getConnection();
      $id_exists = $connection->select('bazaarvoice_productfeed_custom_attributes', 'bpfca')
        ->condition('attribute', $attribute_id)
        ->countQuery()
        ->execute()->fetchField();

      // Id already exists in the Database?
      if ($id_exists) {
        $form_state->setErrorByName('attribute_id', t('A custom attribute with that ID already exists'));
      }
    }
    // No attribute ID or invalid attribute ID.
    else {
      $form_state->setErrorByName('attribute_id', t('Invalid attribute ID provided'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();

    module_load_include('inc', 'bazaarvoice_productfeed', 'includes/attributes');

    $attribute_id = Html::escape(trim(strip_tags($values['attribute_id'])));
    $values = array(
      'label' => Html::escape(trim(strip_tags($values['attribute_label']))),
      'field_types' => array(),
    );
    // Loop through the submitted field types.
    foreach ($values['field_types'] as $value => $state) {
      // Was the field type selected?
      if ($value === $state) {
        $values['field_types'][] = $value;
      }
    }

    bazaarvoice_productfeed_set_custom_attribute($attribute_id, $values);

    // Alert that attribute was created.
    drupal_set_message(t('Attribute "@attribute" created.', array('@attribute' => $values['label'])), 'status', TRUE);

    // Redirect to the attributes admin page.
    $response = new RedirectResponse('/admin/config/services/bazaarvoice/productfeed/attributes');
    $response->send();
  }

}
