<?php

namespace Drupal\bazaarvoice_productfeed\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Component\Utility\Html;

/**
 * Class AttributesAddForm.
 *
 * @package Drupal\bazaarvoice_productfeed\Form
 */
class AttributesEditForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'bazaarvoice_productfeed_attributes_edit_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $bv_attribute = NULL) {
    $values = $form_state->getValues();

    // Set default values based on attribute.
    if (!isset($values['attribute_label'])) {
      $values['attribute_label'] = $bv_attribute['label'];
    }
    if (!isset($values['attribute_id'])) {
      $values['attribute_id'] = $bv_attribute['attribute'];
    }
    if (!isset($values['field_types'])) {
      $values['field_types'] = $bv_attribute['field_types'];
    }
    $form['#productfeed_attribute'] = $bv_attribute;
    $form['attribute_label'] = [
      '#type' => 'textfield',
      '#title' => t('Label'),
      '#required' => TRUE,
      '#description' => t('The human readable label'),
      '#default_value' => isset($values['attribute_label']) ? $values['attribute_label'] : '',
    ];
    $form['attribute_id'] = [
      '#type' => 'textfield',
      '#title' => t('Attribute ID'),
      '#required' => TRUE,
      '#description' => t('The ID value to be used in the Product Feed. Once set this cannot be changed.'),
      '#default_value' => isset($values['attribute_id']) ? $values['attribute_id'] : '',
    ];
    $field_options = ['text' => $this->t('Text')];

    // Get list of all field types.
    $field_type_definitions = \Drupal::service('plugin.manager.field.field_type')
      ->getDefinitions();
    foreach ($field_type_definitions as $field_type_key => $field_type) {
      if (!isset($field_options[$field_type_key])) {
        $field_options[$field_type_key] = $field_type['label'];
      }
    }

    $form['field_types'] = array(
      '#type' => 'checkboxes',
      '#title' => t('Field Types'),
      '#required' => TRUE,
      '#description' => t('What field types can be mapped to this attribute'),
      '#options' => $field_options,
    );
    // Have a value passed?
    if (isset($values['field_types'])) {
      $form['field_types']['#default_value'] = array_filter(unserialize($values['field_types']));
    }
    // Have field options?
    if (count($field_options) == 0) {
      $form['field_types']['#type'] = 'markup';
      $form['field_types']['#value'] = t('There are no field types available.');
    }

    $form['actions']['#type'] = 'actions';
    $form['actions']['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Update Attribute'),
    );

    // Disable the ID field.
    $form['attribute_id']['#disabled'] = TRUE;

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();

    // Check that the label is a plain text label.
    $label = Html::escape(trim(strip_tags($values['attribute_label'])));
    // Label not provided?
    if (!$label) {
      $form_state->setErrorByName('attribute_label', t('Invalid label provided'));
    }
    // Check that the ID is not already in use.
    $bv_attribute_id = Html::escape(trim(strip_tags($values['attribute_id'])));
    // No attribute id?
    if (!$bv_attribute_id) {
      $form_state->setErrorByName('attribute_id', t('Invalid attribute ID provided.'));
    }
    // Else, attempting to change the ID?
    elseif ($bv_attribute_id != $form['#productfeed_attribute']['attribute']) {
      $form_state->setErrorByName('attribute_id', t('Attribute ID cannot be changed.'));
    }

  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();

    module_load_include('inc', 'bazaarvoice_productfeed', 'includes/attributes');

    // Get attribute id.
    $bv_attribute_id = Html::escape(trim(strip_tags($values['attribute_id'])));
    // Prep values.
    $values = array(
      'label' => Html::escape(trim(strip_tags($values['attribute_label']))),
      'field_types' => array(),
    );

    $field_types = array_filter($form_state->getValue(['field_types']));
    sort($field_types);
    $values['field_types'] = $field_types;

    bazaarvoice_productfeed_set_custom_attribute($bv_attribute_id, $values);
    // Set message about updated attribute.
    drupal_set_message(t('Attribute "@attribute" updated.', array('@attribute' => $values['label'])));

  }

}
