<?php

namespace Drupal\bazaarvoice_productfeed\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Class SettingsForm.
 *
 * @package Drupal\bazaarvoice_productfeed\Form
 */
class SettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'bazaarvoice_productfeed_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'bazaarvoice_productfeed.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $config = $this->config('bazaarvoice_productfeed.settings');

    $form['feed'] = array(
      '#type' => 'fieldset',
      '#title' => t('Product Feed Settings'),
      '#description' => t('Settings generating a product feed.'),
      '#collapsible' => TRUE,
    );
    $form['feed']['feed_location'] = array(
      '#type' => 'textfield',
      '#title' => t('File path'),
      '#description' => t('Directory within files directory to save product feed files. (relative to files directory). <br /> Do not include beginning or trailing slash. If the directory does not exist it will be created.'),
      '#default_value' => $config->get('feed_location'),
    );
    $form['feed']['feed_file_prefix'] = array(
      '#type' => 'textfield',
      '#title' => t('Filename prefix'),
      '#description' => t('The prefix for the product feed filename. Can only contain letters, numbers, underscores and hyphens. The date will always be appended to the filename. This is useful if you have multiple sites uploading feed files to your Bazaarvoice FTP location.'),
      '#default_value' => $config->get('feed_file_prefix'),
    );

    $form['feed']['site_name'] = array(
      '#type' => 'textfield',
      '#title' => t('Site Name'),
      '#description' => t('Site name to pass in the product feed.'),
      '#maxlength' => 255,
      '#default_value' => $config->get('site_name'),
    );
    $form['feed']['feed_name'] = array(
      '#type' => 'textfield',
      '#title' => t('Feed Name'),
      '#maxlength' => 255,
      '#description' => t('Feed name, supplied by Bazaarvoice'),
      '#default_value' => $config->get('feed_name'),
    );

    $form['ftp'] = array(
      '#type' => 'fieldset',
      '#title' => t('Product Feed SFTP Settings'),
      '#description' => t('Settings for connecting to the Bazaarvoice SFTP for uploading product data. It is recommended not configuring these values in this form, instead configure them using $conf variables in your settings.php'),
      '#collapsible' => TRUE,
    );
    $form['ftp']['username'] = array(
      '#type' => 'textfield',
      '#title' => t('FTP Username'),
      '#description' => t('Username to connect to the SFTP server.'),
      '#default_value' => $config->get('username'),
    );
    $form['ftp']['password'] = array(
      '#type' => 'password',
      '#title' => t('FTP Password'),
      '#description' => t('Password for connecting to the SFTP server.'),
      '#default_value' => $config->get('password'),
    );
    $form['ftp']['frequency'] = array(
      '#type' => 'select',
      '#title' => t('SFTP Upload Frequency'),
      '#description' => t('Select frequency of SFTPing Product Feed to Bazaarvoice. Timing may not be exact as it is limited to the frequency of when cron is run.'),
      '#options' => array(
        0 => t('Never'),
        3600 => t('Hourly'),
        43200 => t('Twice a Day'),
        86400 => t('Daily'),
        604800 => t('Weekly'),
      ),
      '#default_value' => $config->get('frequency'),
    );

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->get(['feed', 'ftp']);

    // Get the file location selected.
    if ($directory = $values['bazaarvoice_feed_location']) {
      $directory_safe = Html::escape(trim(str_replace(' ', '-', $directory)));
      if ($directory_safe != $directory) {
        $form_state->setErrorByName('bazaarvoice_feed_location', t('Invalid directory name provided.'));
      }
      else {
        $directory = file_default_scheme() . '://' . $directory;
        // Check that directory exists and is writable.
        if (!file_prepare_directory($directory, FILE_CREATE_DIRECTORY | FILE_MODIFY_PERMISSIONS)) {
          $form_state->setErrorByName('bazaarvoice_feed_location', t('File directory does not exist or is not writable.'));
        }
      }
    }

    if ($file_prefix = $values['bazaarvoice_feed_file_prefix']) {
      if (preg_match('@[^a-zA-Z0-9_\-]+@', $file_prefix)) {
        $form_state->setErrorByName('bazaarvoice_feed_file_prefix', t('Filename prefix can only contain letters, numbers, underscores and hyphens.'));
      }
    }

    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();

    foreach ($values as $key => $value) {
      if (!in_array($key, [
        'feed_location',
        'feed_file_prefix',
        'site_name',
        'feed_name',
        'username',
        'password',
        'frequency',
      ])) {
        continue;
      }
      $this->configFactory()
        ->getEditable('bazaarvoice_productfeed.settings')
        ->set($key, $value)
        ->save();
    }

    parent::submitForm($form, $form_state);
  }

}
