<?php

/**
 * @file
 * Provides the Bazaarvoice Reviews reviews interface.
 */

use Drupal\Core\Render\Element;
use Drupal\Core\Url;
use Drupal\Component\Utility\Html;
use Drupal\Core\Entity\EntityInterface;
use Drupal\bazaarvoice\Bazaarvoice;

/**
 * Create content for the write a review page for a product based on method.
 *
 * @param string $type
 *   The entity type.
 * @param object $entity
 *   The entity object.
 *
 * @return string
 *   Generated content HTML.
 */
function bazaarvoice_reviews_new_review($type, EntityInterface $entity) {
  module_load_include('module', 'bazaarvoice_products');
  $content = '';
  // Attempt to get product info.
  if ($product_info = bazaarvoice_products_get_product_info($type, $entity)) {
    // Get the method for writing a review.
    $method = \Drupal::config('bazaarvoice_reviews.settings')
      ->get('method');
    // Bazaarvoice hosted form?
    if ($method == 'hosted') {
      $content = bazaarvoice_reviews_new_hosted_review($product_info);
    }
    // Else assume self hosted.
    else {
      $content = bazaarvoice_reviews_new_conversations_review($product_info);
    }
  }

  return $content;
}

/**
 * Return content for hosted write a review form.
 *
 * @param array $product_info
 *    Array of product entity information.
 *
 * @return string
 *   HTML output for the page.
 */
function bazaarvoice_reviews_new_hosted_review(array $product_info) {
  // Add the external Bazaarvoice js url.
  $hosted_js_url = bazaarvoice_reviews_get_hosted_js_url();
  // Get submission url.
  $submission_url = bazaarvoice_reviews_get_hosted_submission_url($product_info['type'], $product_info['eid']);
  // Get user token.
  $user_token = bazaarvoice_reviews_get_user_token();

  // Render element.
  $review = [
    '#theme' => 'bazaarvoice_reviews_new_hosted_review',
    '#form_container' => '<div id="BVSubmissionContainer"></div>',
    '#attached' => [
      'library' => [
        'bazaarvoice_reviews/bv_reviews',
      ],
      // @see https://www.drupal.org/developing/api/8/assets.
      // Access the above using drupalSettings.bv_reviews.hosted_js_url.
      'drupalSettings' => [
        'bv_reviews' => [
          'hosted_js_url' => $hosted_js_url,
          'submissionContainerUrl' => $submission_url,
          'submission_container' => ['userToken' => $user_token],
        ],
      ],
    ],
  ];

  return $review;
}

/**
 * Build specific url for external Bazaarvoice javascript.
 *
 * @return string
 *   the url to the external javascript.
 */
function bazaarvoice_reviews_get_hosted_js_url() {
  $bvu_config = \Drupal::config('bazaarvoice_ui.settings');
  $bvr_config = \Drupal::config('bazaarvoice_reviews.settings');

  $url = 'http' . ($bvr_config->get('secure') ? 's' : '');
  $url .= '://';
  $url .= $bvu_config->get('client_code') . '.ugc.bazaarvoice.com/';
  $url .= $bvu_config->get('mode') == 'stg' ? 'bvstaging/' : '';
  $url .= 'static/' . $bvr_config->get('display_id') . '-';
  $url .= bazaarvoice_get_locale(FALSE, TRUE) . '/';
  $url .= 'bvapi.js';
  return $url;
}

/**
 * Generate a url to the write a review page for a product.
 *
 * @param string $type
 *   The entity type.
 * @param string $eid
 *   The entity id.
 *
 * @return string
 *   Absolute URL to write-a-review page for the product.
 */
function bazaarvoice_reviews_get_hosted_submission_url($type, $eid) {
  $write_review_uri = bazaarvoice_reviews_get_write_a_review_uri();
  $uri = $type . '/' . $eid . $write_review_uri;
  return Url::fromUri('internal:/' . $uri)->toString();
}

/**
 * Generate a unique user token.
 *
 * @return string
 *   The generated user token.
 */
function bazaarvoice_reviews_get_user_token() {
  /* @var $user \Drupal\Core\Session\AccountProxyInterface */
  $user = \Drupal::currentUser();

  $user_token = '';
  // Logged-in user with data?
  if ($user->id()) {
    // Get the shared key.
    $shared_key = \Drupal::config('bazaarvoice_reviews.settings')
      ->get('shared_key');
    // Build user token.
    $user_token = array(
      'date=' . date('Ymd'),
      'userid=' . $user->id(),
      'emailaddress=' . urlencode($user->getEmail()),
    );
    // Implode token, joint by '&'.
    $user_token = implode('&', $user_token);
    // Hash the user token.
    $user_token = md5($shared_key . $user_token) . bin2hex($user_token);
  }

  return $user_token;
}

/**
 * Return themed output for writing a review page for Conversations API.
 *
 * @param array $product_info
 *   Product entity information.
 *
 * @return string
 *   HTML for review page.
 */
function bazaarvoice_reviews_new_conversations_review(array $product_info) {
  // Get the form.
  $form = array(
    '#theme' => 'bazaarvoice_reviews_new_conversations_review',
    '#form' => \Drupal::formBuilder()
      ->getForm('Drupal\bazaarvoice_reviews\Form\BazaarvoiceReviewsAddReviewForm', $product_info),
    '#product_info' => $product_info,
  );

  return $form;
}

/**
 * Wrapper to submit review data to Bazaarvoice.
 *
 * @param string $product_id
 *   The id of the product to submit a review for.
 * @param array $values
 *   Values to submit as part of the review.
 *
 * @return array|bool
 *   Array of submission data or boolean FALSE.
 */
function bazaarvoice_reviews_submit_conversation_review($product_id, array $values) {
  $submit = FALSE;
  // Create new bazaarvoiceReview object.
  $bazaarvoice_review = new BazaarvoiceReview();
  // Get the locale.
  $locale = bazaarvoice_get_locale(FALSE, TRUE);
  // Store field values.
  $field_values = array();
  // Get the fields.
  if ($fields = $bazaarvoice_review->getFormFields($product_id, $locale)) {
    // Loop through each field and validate.
    foreach ($fields as $id => $field) {
      $value = NULL;
      // Get the field value.
      if (isset($values[$id])) {
        $value = $values[$id];
      }

      if (!is_null($value)) {
        $field_values[$id] = $value;
        // Validate the value.
        if ($field_errors = $bazaarvoice_review->getFieldErrors($field, $value)) {
          $errors[$id] = $field_errors;
        }
      }
    }
  }

  // No errors?
  if (empty($errors)) {
    // Add device fingerprint.
    $submitted_fields['fp'] = isset($values['fp']) ? $values['fp'] : '';
    // Have bazaarvoice submit.
    if ($data = $bazaarvoice_review->submitReview($product_id, $locale, $field_values)) {
      $submit = array('success' => t('Your review has been submitted.'));
      // Know typical hours till display?
      if (isset($data['TypicalHoursToPost'])) {
        // Add to success message.
        $submit['success'] .= ' ' . t('Reviews can take up to @hours hours to display.', array('@hours' => $data['TypicalHoursToPost']));
      }
    }
    else {
      // Get the errors.
      if ($review_errors = $bazaarvoice_review->getApiErrors()) {
        foreach ($review_errors['FieldErrors'] as $field => $error) {
          $errors[$field] = array($error['Message']);
        }
        foreach ($review_errors['Errors'] as $error) {
          $errors[] = $error;
        }
        if (!empty($errors)) {
          $submit = array('errors' => $errors);
        }
      }
      // Else, some other error occurred.
      else {
        $submit = array('errors' => array('' => t('There is an error with the data you are trying to submit.')));
      }
    }
  }
  else {
    $submit = array('errors' => $errors);
  }

  return $submit;
}

/**
 * Return array of Drupal form fields for conversations review form.
 *
 * @param string $product_id
 *   The product to retrieve the fields for.
 *
 * @return mixed
 *   Array ofr Drupal form elements.
 */
function bazaarvoice_reviews_get_conversations_drupal_fields($product_id) {
  // Create a new review object.
  $bazaarvoice_review = new BazaarvoiceReview();
  // Get the locale.
  $locale = bazaarvoice_get_locale(FALSE, TRUE);
  // Retrieve the form fields.
  return $bazaarvoice_review->getDrupalFormFields($product_id, $locale);
}

/**
 * Set the default values for fields from previous values.
 *
 * @param array $fields
 *   Array of form fields to set default values.
 * @param array $previous_values
 *   Array of previously submitted values.
 */
function bazaarvoice_reviews_set_conversations_field_values(array &$fields, array $previous_values) {
  foreach ($fields as $field_id => &$field) {
    // Field Id an attribute from a recursive call?
    if (substr($field_id, 0, 1) == '#') {
      // Skip it.
      continue;
    }
    // Field has a previous value?
    if (isset($previous_values[$field_id])) {
      // Set it as the default.
      $field['#default_value'] = $previous_values[$field_id];
    }
    // Does field have children?
    if (Element::children($field)) {
      // Recursive call for children.
      bazaarvoice_reviews_set_conversations_field_values($field, $previous_values);
    }
  }
}

/**
 * Check for errors in submitted values for Bazaarvoice fields.
 *
 * @param string $product_id
 *   The id of the current product review is being written for.
 * @param array $values
 *   The values being submitted for the review fields.
 *
 * @return array|bool
 *   Array of field error messages or boolean FALSE.
 */
function bazaarvoice_reviews_conversations_field_errors($product_id, array $values) {
  $errors = array();
  // Create a new review object.
  $bazaarvoice_review = new BazaarvoiceReview();
  // Get the locale.
  $locale = bazaarvoice_get_locale(FALSE, TRUE);
  // Store field values.
  $field_values = array();
  // Get the fields.
  if ($fields = $bazaarvoice_review->getFormFields($product_id, $locale)) {
    // Loop through each field and validate.
    foreach ($fields as $id => $field) {
      $value = NULL;
      // Get the field value.
      if (isset($values[$id])) {
        $value = $values[$id];
      }

      if (!is_null($value)) {
        $field_values[$id] = $value;
        // Validate the value.
        if ($field_errors = $bazaarvoice_review->getFieldErrors($field, $value)) {
          $errors[$id] = $field_errors;
        }
      }
    }
  }

  // No errors yet?
  if (empty($errors)) {
    // Have bazaarvoice validate.
    if (!$bazaarvoice_review->validateReview($product_id, $locale, $field_values)) {
      // Get the errors.
      if ($review_errors = $bazaarvoice_review->getApiErrors()) {
        foreach ($review_errors['FieldErrors'] as $field => $error) {
          $errors[$field] = array($error['Message']);
        }
      }
      // Else, some other error occurred.
      else {
        $errors = array('' => t('There is an error with the data you are trying to submit.'));
      }
    }
  }

  return empty($errors) ? FALSE : $errors;
}
