<?php

namespace Drupal\bazaarvoice_reviews\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\Core\Link;
use Drupal\Component\Utility\UrlHelper;

/**
 * Contains Bazaarvoice Reviews Settings Form.
 *
 * @package Drupal\bazaarvoice_reviews\Form
 */
class BazaarvoiceReviewsSettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'bazaarvoice_reviews_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['bazaarvoice_reviews.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $config = $this->config('bazaarvoice_reviews.settings');

    $form['settings'] = array(
      '#type' => 'fieldset',
      '#title' => $this->t('Product Review Settings'),
      '#description' => $this->t('Settings for submitting product reviews'),
    );

    $form['settings']['enable'] = array(
      '#type' => 'checkbox',
      '#title' => $this->t('Enable product reviews'),
      '#default_value' => $config->get('enable'),
    );

    $form['settings']['secure'] = array(
      '#type' => 'checkbox',
      '#title' => $this->t('Use secure page for reviews form.'),
      '#description' => $this->t('If the review submission page should use HTTPS. Requires site to be @link', [
        '@link' => Link::fromTextandURL(
          t('configured to use HTTPS.'),
          Url::fromUri('https://drupal.org/https-information'))->toString()
      ]),
      '#default_value' => $config->get('secure'),
    );

    $form['settings']['method'] = array(
      '#type' => 'select',
      '#title' => $this->t('Reviews Method'),
      '#description' => $this->t('Select which method to use for displaying review form and submitting reviews.'),
      '#options' => array(
        'hosted' => $this->t('Hosted UI'),
        'conversation' => $this->t('Conversations API (self hosted form)'),
      ),
      '#default_value' => $config->get('method'),
    );

    $form['settings']['hosted'] = array(
      '#type' => 'fieldset',
      '#title' => $this->t('Hosted UI settings'),
      '#description' => $this->t('Settings needed for Bazaarvoice hosted reviews'),
      '#states' => array(
        'visible' => array(
          ':input[name="bazaarvoice_reviews_method"]' => array('value' => 'hosted'),
        ),
      ),
    );

    $form['settings']['hosted']['url'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Reviews Base URL'),
      '#description' => $this->t('Base url for writing reviews, supplied by Bazaarvoice. ex: reviews.client.com (do not include http(s)://)'),
      '#default_value' => $config->get('url'),
    );

    $form['settings']['hosted']['display_id'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Display Code'),
      '#description' => $this->t('Display code for writing reviews. Supplied by Bazaarvoice.'),
      '#default_value' => $config->get('display_id'),
    );

    $form['settings']['hosted']['shared_key'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Shared Key'),
      '#default_value' => $config->get('shared_key'),
    );

    $form['url_settings'] = array(
      '#type' => 'fieldset',
      '#title' => $this->t('Review URL Settings'),
      '#description' => $this->t('Settings for the urls for review pages.'),
    );

    $form['url_settings']['reviews_uri'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Reviews path'),
      '#description' => $this->t('Path for viewing the reviews for a product entity. Relative to the entity page.'),
      '#default_value' => bazaarvoice_reviews_get_reviews_uri(),
    );

    $form['url_settings']['write_uri'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Write A Review path'),
      '#description' => $this->t('Path for the write a review page, relative to the entity page.'),
      '#required' => TRUE,
      '#default_value' => bazaarvoice_reviews_get_write_a_review_uri(),
    );

    $form['display'] = array(
      '#type' => 'fieldset',
      '#title' => $this->t('Display Settings'),
    );

    $form['display']['per_page'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Reviews Per Page'),
      '#description' => $this->t('How many reviews to display per page. (Cannot exceed 100)'),
      '#default_value' => $config->get('per_page'),
    );

    $form['cache'] = array(
      '#type' => 'fieldset',
      '#title' => $this->t('Cache Settings'),
    );

    $form['cache']['cache'] = array(
      '#type' => 'select',
      '#title' => $this->t('Review Cache Lifetime'),
      '#description' => $this->t('Amount of time to cache reviews for a product before calling from API.'),
      '#options' => array(
        0 => $this->t('Never'),
        3600 => $this->t('Hourly'),
        43200 => $this->t('Twice a Day'),
        86400 => $this->t('Daily'),
        604800 => $this->t('Weekly'),
      ),
      '#default_value' => $config->get('cache'),
    );

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();

    $per_page = $values['per_page'];

    if (!is_numeric($per_page) || !is_int($per_page + 0) || ($per_page < 0)) {
      $form_state->setErrorByName('per_page', t('Must Provide an integer value greater than 0.'));
    }
    elseif ($per_page > 100) {
      $form_state->setErrorByName('per_page', t('Must Provide an integer value less then 100.'));
    }

    // Make sure value is safe for use in a url.
    if (!UrlHelper::isValid($values['reviews_uri'])) {
      $form_state->setErrorByName('reviews_uri', t('Reviews path is not a valid uri string.'));
    }

    // Make sure value is safe for use in a url.
    if (!UrlHelper::isValid($values['write_uri'])) {
      $form_state->setErrorByName('write_uri', t('Write a Review path is not a valid uri string.'));
    }

    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    $values = $form_state->getValues();
    $keys = [
      'enable',
      'secure',
      'method',
      'hosted',
      'url',
      'display_id',
      'shared_key',
      'write_uri',
      'reviews_uri',
      'per_page',
      'cache',
    ];

    foreach ($keys as $key) {
      if (isset($values[$key])) {
        $this->configFactory()->getEditable('bazaarvoice_reviews.settings')
          ->set($key, $values[$key])
          ->save();
      }
    }

    parent::submitForm($form, $form_state);
  }

}
