<?php

namespace Drupal\bazaarvoice_ui\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Component\Utility\Html;

/**
 * Class BazaarvoiceUiAdminLocaleForm.
 *
 * @package Drupal\bazaarvoice_ui\Form
 */
class BazaarvoiceUiAdminLocaleForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'bazaarvoice_ui_admin_locale_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = [];
    // Build a table for configuring locales.
    $form['locales_set'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Language Locales'),
      '#description' => $this->t('Configure locale codes for each language enabled on the site. Bazaarvoice expects the locale to be formatted as [language code]_[COUNTRY CODE] (example: en_US). Leave empty to delete the locale from the BV DB.'),
    ];

    $form['locales_set']['locales'] = [
      '#theme' => 'bazaarvoice_ui_admin_locale_form_locales',
      '#tree' => TRUE,
    ];

    // Get site languages.
    $languages = bazaarvoice_get_site_languages();
    // Get site locales.
    $locales = bazaarvoice_get_locales();

    // Get default locale.
    $default_locale = '';
    // Get list of default locale options.
    $default_locales = [];

    // Loop through each language.
    foreach ($languages as $lang_code => $language) {
      if (strlen($lang_code) == 2) {
        $language_code = strtolower($lang_code);
        // Try to get a locale code from our stored locales.
        if (isset($locales[$lang_code]['locale'])) {
          $locale_code = strtoupper($locales[$lang_code]['locale']);
        }
      }
      else {
        $language_code = strtolower(substr($lang_code, 0, 2));
        $has_locale_code = 0;
        if (isset($locales[$lang_code]['locale'])) {
          // Get it from the DB values.
          $locale_code = preg_replace('/^.*([a-zA-Z]{2})$/', '$1', $locales[$lang_code]['locale'], -1, $has_locale_code);
        }

        if ($has_locale_code) {
          $locale_code = strtoupper($locale_code);
        }
      }

      $default_locales[$lang_code] = $language;
      $form['locales_set']['locales'][$lang_code] = [
        '#title' => Html::escape(t('@title locale', [
          '@title' => $language,
        ])),
        '#title_display' => 'invisible',
        '#type' => 'textfield',
        '#field_prefix' => $language_code . '_',
        '#description' => $this->t('Provide the uppercase 2 letter country code for this locale.'),
        '#max_length' => 2,
      ];

      if (isset($locale_code)) {
        $form['locales_set']['locales'][$lang_code]['#default_value'] = $locale_code;

        if (isset($locales[$lang_code]['is_default']) && $locales[$lang_code]['is_default'] == TRUE) {
          $default_locale = $lang_code;
        }
      }
      // Clean variables.
      $language_code = '';
      $has_locale_code = FALSE;
      $locale_code = '';
    }

    $form['locales_set']['locales']['locale_default'] = [
      '#type' => 'radios',
      '#title' => $this->t('Default locale'),
      '#title_display' => 'invisible',
      '#options' => $default_locales,
      '#default_value' => $default_locale,
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save Settings'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValue(['locales']);
    $locale_default = (isset($values['locale_default']) ? $values['locale_default'] : NULL);

    // Get site languages.
    $languages = bazaarvoice_get_site_languages();
    // Loop through the submitted locales.
    foreach ($values as $lang_code => $locale) {
      // Is there a locale value?
      if (isset($languages[$lang_code]) && $locale) {
        // Is it not two capital letters?
        if (!preg_match('/^[A-Z]{2}$/', $locale)) {
          $form_state->setErrorByName('locales][' . $lang_code, t('Invalid country code provided for @language'));
        }
      }
    }

    // Get default locale language and make sure a value was submitted.
    if (!isset($locale_default)) {
      $form_state->setErrorByName('locales][locale_default', t('Default must have a valid locale code.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValue(['locales']);
    $locale_default = (isset($values['locale_default']) ? $values['locale_default'] : NULL);
    unset($values['locale_default']);

    // Get the enabled site languages.
    $languages = bazaarvoice_get_site_languages();

    // Array to hold locales to create/update.
    $locales = [];
    // Locales to delete.
    $delete = [];

    // Loop through each language.
    foreach ($languages as $lang_code => $language) {
      if (!isset($values[$lang_code]) || empty($values[$lang_code])) {
        $delete[] = $lang_code;
      }
    }

    // Add new languages.
    foreach ($values as $lang_code => $locale) {
      if (isset($locale) && !empty($locale)) {
        $locales[$lang_code] = [
          'language' => $lang_code,
          // @TODO: Decide if BV expects just the country code or whole locale.
          // 'locale' => substr($lang_code, 0, 2) . '_' . $language,
          'locale' => $locale,
          'is_default' => ($locale_default == $lang_code ? '1' : '0'),
        ];
      }
    }

    // Save the locales.
    bazaarvoice_set_locales($locales);

    // Delete unset locales.
    bazaarvoice_delete_locales($delete);

    drupal_set_message(t('Locales have been saved.'));
  }

}
